import io
import os
import queue
import argparse
import sys
from urllib.parse import urlparse
from pathlib import Path
import shlex
import getpass
import subprocess


from cmd2 import (
    Cmd,
    style,
    with_argparser,
    Statement,
    Settable,
    with_default_category,
)
import click
from faraday_plugins.plugins.manager import (
    PluginsManager,
    ReportAnalyzer,
    CommandAnalyzer,
)
from simple_rest_client.exceptions import ClientConnectionError
from tabulate import tabulate

from simple_rest_client.exceptions import ClientError

from faraday_cli.extras.halo.halo import Halo
from faraday_cli.config import active_config
from faraday_cli.shell import utils

from faraday_cli.api_client import FaradayApi
from faraday_cli.api_client.exceptions import InvalidCredentials, Invalid2FA
from faraday_cli.shell import modules  # noqa: F401

logo = """
    ______                     __               _________
   / ____/___ __________ _____/ /___ ___  __   / ____/ (_)
  / /_  / __ `/ ___/ __ `/ __  / __ `/ / / /  / /   / / /
 / __/ / /_/ / /  / /_/ / /_/ / /_/ / /_/ /  / /___/ / /
/_/    \__,_/_/   \__,_/\__,_/\__,_/\__, /   \____/_/_/
                                   /____/
"""  # noqa: W605


@with_default_category("Core")
class FaradayShell(Cmd):
    prompt = "Faraday> "
    intro = "Welcome to Faraday Cli! Type ? to list commands"
    doc_header = "Available Commands (type help <command>)"
    ruler = "-"

    emojis = {
        "cross": "\U0000274c",
        "check": "\U00002714",
        "arrow_up": "\U00002b06",
        "page": "\U0001f4c4",
        "laptop": "\U0001f4bb",
    }
    delattr(Cmd, "do_run_pyscript")
    delattr(Cmd, "do_run_script")

    def __init__(self, *args, **kwargs):
        super().__init__(
            persistent_history_file="~/.faraday-cli_history", *args, **kwargs
        )
        self.hidden_commands += ["EOF", "cd", "alias"]
        self.run_as_shell = False
        self.TABLE_PRETTY_FORMAT = "psql"
        # hide unwanted settings
        settings_to_hide = ["debug"]
        for setting_name in settings_to_hide:
            self.remove_settable(setting_name)
        intro = [style(logo, fg="red")]
        if active_config.faraday_url and active_config.token:
            intro.append(
                style(f"Server: {active_config.faraday_url}", fg="green")
            )
            self.api_client = FaradayApi(
                active_config.faraday_url,
                ignore_ssl=active_config.ignore_ssl,
                token=active_config.token,
            )
        else:
            self.api_client = FaradayApi()
            intro.append(
                style("Missing faraday server, run 'auth'", fg="yellow")
            )
        self.intro = "\n".join(intro)
        self.data_queue = queue.Queue()
        self.custom_plugins_path = None
        self.update_prompt()
        self.add_settable(
            Settable(
                "custom_plugins_path",
                str,
                "Path of custom plugins",
                onchange_cb=self._onchange_custom_plugins_path,
            )
        )
        self.plugins_manager = PluginsManager(
            active_config.custom_plugins_folder
        )
        self.report_analyzer = ReportAnalyzer(self.plugins_manager)
        self.command_analyzer = CommandAnalyzer(self.plugins_manager)

    def _onchange_custom_plugins_path(self, param_name, old, new):
        custom_plugins_path = Path(new)
        if custom_plugins_path.is_dir():
            active_config.custom_plugins_path = new
            active_config.save()

    auth_parser = argparse.ArgumentParser()
    auth_parser.add_argument(
        "-f",
        "--faraday-url",
        type=str,
        help="Faraday server URL",
        required=False,
    )
    auth_parser.add_argument(
        "-i",
        "--ignore-ssl",
        action="store_true",
        help="Ignore SSL verification",
    )
    auth_parser.add_argument(
        "-u", "--user", type=str, help="Faraday user", required=False
    )
    auth_parser.add_argument(
        "-p", "--password", type=str, help="Faraday password", required=False
    )

    @with_argparser(auth_parser)
    def do_auth(self, args):
        """Authenticate with a faraday server"""
        faraday_url = args.faraday_url
        user = args.user
        password = args.password
        ignore_ssl = args.ignore_ssl
        if not faraday_url:
            faraday_url = utils.validate_url(
                click.prompt(
                    "\nFaraday url", default=active_config.faraday_url
                )
            )
            url_data = urlparse(faraday_url)
            if url_data.scheme == "https":
                ignore_ssl = (
                    click.prompt(
                        f"\nValidate SSL certificate for [{faraday_url}]",
                        type=click.Choice(
                            choices=["Y", "N"], case_sensitive=False
                        ),
                        default="Y",
                    )
                    == "N"
                )
        else:
            faraday_url = utils.validate_url(faraday_url)
        if not user:
            user = click.prompt("\nUser")
        if not password:
            password = click.prompt("\nPassword", hide_input=True)
        try:
            api_client = FaradayApi(faraday_url, ignore_ssl=ignore_ssl)
            login_ok = api_client.login(user, password)
            if login_ok is None or login_ok is True:
                if login_ok is None:
                    # 2FA Required
                    second_factor = click.prompt("2FA")
                else:
                    second_factor = None
                token = api_client.get_token(second_factor)
                active_config.faraday_url = faraday_url
                active_config.ignore_ssl = args.ignore_ssl
                active_config.token = token
                active_config.workspace = None
                active_config.save()
                self.api_client = FaradayApi(
                    faraday_url, ignore_ssl=ignore_ssl, token=token
                )
                self.poutput(style("Saving config", fg="green"))
                self.poutput(
                    style(
                        f"{self.emojis['check']} Authenticated with faraday: {faraday_url}",  # noqa: E501
                        fg="green",
                    )
                )
                self.update_prompt()
            else:
                self.perror("Invalid credentials")
                if self.run_as_shell:
                    sys.exit(1)
        except Invalid2FA:
            self.perror("Invalid 2FA")
            if self.run_as_shell:
                sys.exit(1)
        except InvalidCredentials:
            self.perror("Invalid credentials")
            if self.run_as_shell:
                sys.exit(1)
        except ClientError:
            self.perror("Invalid credentials")
            if self.run_as_shell:
                sys.exit(1)
        except ClientConnectionError as e:
            self.perror(f"Connection refused: {e} (check your Faraday server)")
            if self.run_as_shell:
                sys.exit(1)
        except Exception as e:
            self.perror(f"{e}")
            if self.run_as_shell:
                sys.exit(1)

    def do_exit(self, inp):
        """Exit shell"""
        self.poutput(style("Bye", fg="green"))
        return True

    def help_exit(self):
        print("exit the application. Shorthand: Ctrl-D.")

    def postcmd(self, stop, line):
        def send_to_faraday(ws, data):
            spinner = Halo(text="Sending", text_color="green", spinner="dots")
            spinner.start()
            self.api_client.bulk_create(ws, data)
            spinner.stop()

        while not self.data_queue.empty():
            data = self.data_queue.get()
            message = f"{self.emojis['arrow_up']} Sending data to workspace: {data['workspace']}"  # noqa: E501
            self.poutput(style(message, fg="green"))
            send_to_faraday(data["workspace"], data["json_data"])
            self.poutput(style(f"{self.emojis['check']} Done", fg="green"))

        return Cmd.postcmd(self, stop, line)

    def update_prompt(self) -> None:
        self.prompt = self.get_prompt()

    def get_prompt(self) -> str:
        if active_config.workspace:
            return style(f"[ws:{active_config.workspace}]> ", fg="blue")
        else:
            return style("Faraday> ", fg="blue")

    status_parser = argparse.ArgumentParser()
    status_parser.add_argument(
        "-p", "--pretty", action="store_true", help="Pretty Tables"
    )

    @with_argparser(status_parser)
    def do_status(self, args):
        """Show Cli status"""
        valid_token = self.api_client.is_token_valid()
        version = "-"
        if valid_token:
            version_data = self.api_client.get_version()
            version = f"{version_data['product']}-{version_data['version']}"
        data = [
            {
                "FARADAY SERVER": active_config.faraday_url,
                "IGNORE SSL": active_config.ignore_ssl,
                "VERSION": version,
                "VALID TOKEN": "\U00002714" if valid_token else "\U0000274c",
                "WORKSPACE": active_config.workspace,
            }
        ]
        self.poutput(
            style(
                tabulate(
                    data,
                    headers="keys",
                    tablefmt=self.TABLE_PRETTY_FORMAT
                    if args.pretty
                    else "simple",
                ),
                fg="green",
            )
        )

    def run_command(self, plugin, user, command):
        current_path = os.path.abspath(os.getcwd())
        modified_command = plugin.processCommandString(
            getpass.getuser(), current_path, command
        )
        if modified_command:
            command = modified_command
        p = subprocess.Popen(
            shlex.split(command),
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
        )
        output = io.StringIO()
        while True:
            retcode = p.poll()
            line = p.stdout.readline().decode("utf-8")
            sys.stdout.write(line)
            output.write(line)
            if retcode is not None:
                extra_lines = map(
                    lambda x: x.decode("utf-8"), p.stdout.readlines()
                )
                sys.stdout.writelines(line)
                output.writelines(extra_lines)
                break
        output_value = output.getvalue()
        if retcode == 0:
            plugin.processOutput(output_value)
            return plugin.get_data()
        else:
            return None

    def do_cd(self, path):
        """Change directory"""
        if not path:
            path = "~"
        cd_path = os.path.expanduser(path)
        if os.path.isdir(cd_path):
            os.chdir(cd_path)
        else:
            self.poutput(f"cd: no such file or directory: {path}")

    def default(self, statement: Statement):
        if not active_config.workspace:
            self.perror("No active Workspace")
            os.system(statement.raw)
        else:
            plugin = self.command_analyzer.get_plugin(statement.raw)
            if plugin:
                click.secho(
                    f"{self.emojis['laptop']} Processing {plugin.id} command",
                    fg="green",
                )
                command_json = self.run_command(
                    plugin, getpass.getuser(), statement.raw
                )
                if not command_json:
                    click.secho(
                        f"{self.emojis['cross']} Command execution error!!",
                        fg="red",
                    )
                else:
                    self.data_queue.put(
                        {
                            "workspace": active_config.workspace,
                            "json_data": command_json,
                        }
                    )
            else:
                os.system(statement.raw)

    # do_EOF = do_exit
    # help_EOF = help_exit



import argparse
import json
from collections import defaultdict
from datetime import datetime
import dateutil.parser

from cmd2 import with_argparser, with_default_category, CommandSet
from faraday_cli.extras.halo.halo import Halo
from faraday_cli.extras.termgraph import termgraph
from faraday_cli.shell.utils import (
    SEVERITY_COLORS,
    IGNORE_SEVERITIES,
    SEVERITIES,
)
from faraday_cli.config import active_config
from faraday_cli.api_client.filter import FaradayFilter


@with_default_category("Stats")
class StatsCommands(CommandSet):
    def __init__(self):
        super().__init__()

    stats_parser = argparse.ArgumentParser()
    stats_parser.add_argument(
        "stat_type",
        type=str,
        choices=["severity", "vulns", "date"],
        help="Type of stat",
    )
    stats_parser.add_argument(
        "-w", "--workspace-name", type=str, help="Workspace"
    )
    stats_parser.add_argument(
        "--ignore-info",
        action="store_true",
        help=f"Ignore {'/'.join(IGNORE_SEVERITIES)} vulnerabilities",
    )
    stats_parser.add_argument(
        "--severity",
        type=str,
        help=f"Filter by severity {'/'.join(SEVERITIES)}",
        default=[],
        nargs="*",
    )
    stats_parser.add_argument(
        "--confirmed", action="store_true", help="Confirmed vulnerabilities"
    )

    @with_argparser(stats_parser)
    def do_stats(self, args):
        """Vulns Stats"""
        if not args.workspace_name:
            if active_config.workspace:
                workspace_name = active_config.workspace
            else:
                self._cmd.perror("No active Workspace")
                return
        else:
            workspace_name = args.workspace_name

        def gather_vulns_stats(vulns):
            if vulns["vulnerabilities"]:
                counters = defaultdict(int)
                for vuln in vulns["vulnerabilities"]:
                    if len(vuln["value"]["hostnames"]):
                        host_identifier = vuln["value"]["hostnames"][0]
                    else:
                        host_identifier = vuln["value"]["target"]
                    counters[host_identifier] += 1
                data = list(map(lambda x: [x], counters.values()))
                termgraph_data = termgraph.TERMGRAPH_DATA_TEMPLATE.copy()
                termgraph_data[
                    "title"
                ] = f"Vulnerability stats [{workspace_name}]"
                termgraph_data["data"] = data
                termgraph_data["labels"] = [x for x in counters.keys()]
                termgraph_data["categories"] = ["vulns"]
                termgraph_data["color"] = ["red"]
                return termgraph_data
            else:
                return None

        def gather_severity_stats(vulns):
            if vulns["vulnerabilities"]:
                counters = defaultdict(
                    lambda: {"severity": {x: 0 for x in SEVERITY_COLORS}}
                )
                for vuln in vulns["vulnerabilities"]:
                    if len(vuln["value"]["hostnames"]):
                        host_identifier = vuln["value"]["hostnames"][0]
                    else:
                        host_identifier = vuln["value"]["target"]
                    counters[host_identifier]["severity"][
                        vuln["value"]["severity"]
                    ] += 1
                data = list(
                    map(
                        lambda x: list(x["severity"].values()),
                        counters.values(),
                    )
                )

                termgraph_data = termgraph.TERMGRAPH_DATA_TEMPLATE.copy()
                termgraph_data["title"] = f"Severity stats [{workspace_name}]"
                termgraph_data["data"] = data
                termgraph_data["labels"] = [x for x in counters.keys()]
                termgraph_data["categories"] = list(SEVERITY_COLORS.keys())
                termgraph_data["color"] = list(SEVERITY_COLORS.values())
                termgraph_data["stacked"] = True
                return termgraph_data
            else:
                return None

        def gather_history_stats(vulns):
            if vulns["vulnerabilities"]:
                counters = defaultdict(int)
                DATE_FORMAT = "%Y-%m-%d"
                min_date = datetime.now()
                for vuln in vulns["vulnerabilities"]:
                    vuln_date = dateutil.parser.parse(
                        vuln["value"]["metadata"]["create_time"]
                    )
                    if vuln_date.date() < min_date.date():
                        min_date = vuln_date
                    date_str = vuln_date.strftime(DATE_FORMAT)
                    counters[date_str] += 1
                data = list(map(lambda x: [x], counters.values()))
                termgraph_data = termgraph.TERMGRAPH_DATA_TEMPLATE.copy()
                min_date_str = min_date.strftime(DATE_FORMAT)
                title = f"Heatmap since {min_date_str} [{workspace_name}]"
                termgraph_data["title"] = title
                termgraph_data["data"] = data
                termgraph_data["labels"] = [x for x in counters.keys()]
                termgraph_data["start_dt"] = min_date.strftime(DATE_FORMAT)
                termgraph_data["calendar"] = True
                return termgraph_data
            else:
                return None

        @Halo(text="Gathering data", text_color="green", spinner="dots")
        def graph_stats(gather_data_func, filter_to_apply):
            vulns = self._cmd.api_client.get_vulns(
                workspace, json.dumps(filter_to_apply)
            )
            args_data = gather_data_func(vulns)
            if args_data:
                _, labels, data, colors = termgraph.read_data(args_data)
                if args_data["calendar"]:
                    termgraph.calendar_heatmap(data, labels, args_data)
                else:
                    termgraph.chart(colors, data, args_data, labels)
            else:
                self._cmd.perror(f"No data in workspace {workspace_name}")

        if workspace_name:
            if not self._cmd.api_client.is_workspace_valid(workspace_name):
                self._cmd.perror(f"Invalid workspace: {workspace_name}")
                return
            else:
                workspace = workspace_name
        else:
            if not active_config.workspace:
                self._cmd.perror("No workspace selected")
                return
            else:
                workspace = active_config.workspace
        if args.severity and args.ignore_info:
            self._cmd.perror("Use either --ignore-info or --severity")
            return
        query_filter = FaradayFilter()
        selected_severities = set(map(lambda x: x.lower(), args.severity))
        if selected_severities:
            for severity in selected_severities:
                if severity not in SEVERITIES:
                    self._cmd.perror(f"Invalid severity: {severity}")
                    return
                else:
                    query_filter.require_severity(severity)
        if args.ignore_info:
            for severity in IGNORE_SEVERITIES:
                query_filter.ignore_severity(severity)
        if args.confirmed:
            query_filter.filter_confirmed()
        gather_data_function_choices = {
            "severity": gather_severity_stats,
            "vulns": gather_vulns_stats,
            "date": gather_history_stats,
        }

        graph_stats(
            gather_data_function_choices[args.stat_type],
            query_filter.get_filter(),
        )



import json
import argparse
import sys
from collections import OrderedDict

from cmd2 import style, with_argparser, with_default_category, CommandSet
from tabulate import tabulate
from simple_rest_client.exceptions import NotFoundError

from faraday_cli.extras.halo.halo import Halo
from faraday_cli.config import active_config
from faraday_cli.shell import utils

HOST_CREATE_JSON_SCHEMA = {
    "type": "array",
    "items": {
        "type": "object",
        "properties": {
            "ip": {"type": "string"},
            "description": {"type": "string"},
            "hostnames": {"type": "array"},
        },
        "required": ["ip", "description"],
    },
}


@with_default_category("Hosts")
class HostCommands(CommandSet):
    def __init__(self):
        super().__init__()

    list_host_parser = argparse.ArgumentParser()
    list_host_parser.add_argument(
        "-w", "--workspace-name", type=str, help="Workspace name"
    )
    list_host_parser.add_argument(
        "-j", "--json-output", action="store_true", help="Show output in json"
    )
    list_host_parser.add_argument(
        "-ip", "--show-ip", action="store_true", help="Show ip only"
    )
    list_host_parser.add_argument(
        "-p",
        "--pretty",
        action="store_true",
        help="Show table in a pretty format",
    )
    list_host_parser.add_argument("--port", type=int, help="Listen in port")

    @with_argparser(list_host_parser)
    def do_list_hosts(self, args):
        """List hosts"""

        @Halo(
            text="Gathering data",
            text_color="green",
            spinner="dots",
            stream=sys.stderr,
        )
        def get_data(workspace_name, port_number):
            hosts = self._cmd.api_client.get_hosts(workspace_name, port_number)
            return hosts

        if not args.workspace_name:
            if active_config.workspace:
                workspace_name = active_config.workspace
            else:
                self._cmd.perror("No active Workspace")
                return
        else:
            workspace_name = args.workspace_name
        try:
            hosts = get_data(workspace_name, args.port)
        except NotFoundError:
            self._cmd.perror("Workspace not found")
        else:
            if not hosts["rows"]:
                self._cmd.perror(f"No hosts in workspace: {workspace_name}")
            else:
                if args.json_output:
                    self._cmd.poutput(json.dumps(hosts["rows"], indent=4))
                elif args.show_ip:
                    for host in hosts["rows"]:
                        self._cmd.poutput(host["value"]["ip"])
                else:
                    data = [
                        OrderedDict(
                            {
                                "ID": x["value"]["id"],
                                "IP": x["value"]["ip"],
                                "OS": x["value"]["os"],
                                "HOSTNAMES": ", ".join(
                                    x["value"]["hostnames"]
                                ),
                                "SERVICES": len(
                                    x["value"]["service_summaries"]
                                ),
                                "VULNS": "-"
                                if x["value"]["vulns"] == 0
                                else x["value"]["vulns"],
                            }
                        )
                        for x in hosts["rows"]
                    ]
                    self._cmd.poutput(
                        tabulate(
                            data,
                            headers="keys",
                            tablefmt="psql" if args.pretty else "simple",
                        )
                    )

    get_host_parser = argparse.ArgumentParser()
    get_host_parser.add_argument("host_id", type=int, help="ID of the host")
    get_host_parser.add_argument(
        "-w",
        "--workspace-name",
        type=str,
        help="Workspace name",
        required=False,
    )
    get_host_parser.add_argument(
        "-j", "--json-output", action="store_true", help="Show output in json"
    )
    get_host_parser.add_argument(
        "-p",
        "--pretty",
        action="store_true",
        help="Show table in a pretty format",
    )

    @with_argparser(get_host_parser)
    def do_get_host(self, args):
        """Get host information"""
        if not args.workspace_name:
            if active_config.workspace:
                workspace_name = active_config.workspace
            else:
                self._cmd.perror("No active Workspace")
                return
        else:
            workspace_name = args.workspace_name
        try:
            host = self._cmd.api_client.get_host(workspace_name, args.host_id)
        except NotFoundError:
            self._cmd.perror(
                f"Host ID: {args.host_id} "
                f"in Workspace {workspace_name} not found"
            )
        else:
            if args.json_output:
                self._cmd.poutput(json.dumps(host, indent=4))
            else:
                host_data = [
                    OrderedDict(
                        {
                            "ID": x["id"],
                            "IP": x["ip"],
                            "OS": x["os"],
                            "HOSTNAMES": ", ".join(x["hostnames"]),
                            "OWNER": x["owner"],
                            "OWNED": x["owned"],
                            "VULNS": "-" if x["vulns"] == 0 else x["vulns"],
                        }
                    )
                    for x in [host]
                ]
                self._cmd.poutput("Host:")
                self._cmd.poutput(
                    tabulate(
                        host_data,
                        headers="keys",
                        tablefmt=self._cmd.TABLE_PRETTY_FORMAT
                        if args.pretty
                        else "simple",
                    )
                )
                if host["services"] > 0:
                    services = self._cmd.api_client.get_host_services(
                        workspace_name, args.host_id
                    )
                    services_data = [
                        OrderedDict(
                            {
                                "ID": x["id"],
                                "NAME": x["name"],
                                "DESCRIPTION": x["description"],
                                "PROTOCOL": x["protocol"],
                                "PORT": x["port"],
                                "VERSION": x["version"],
                                "STATUS": x["status"],
                                "VULNS": "-"
                                if x["vulns"] == 0
                                else x["vulns"],
                            }
                        )
                        for x in services
                    ]
                    self._cmd.poutput("\nServices:")
                    self._cmd.poutput(
                        tabulate(
                            services_data,
                            headers="keys",
                            tablefmt=self._cmd.TABLE_PRETTY_FORMAT
                            if args.pretty
                            else "simple",
                        )
                    )
                if host["vulns"] > 0:
                    vulns = self._cmd.api_client.get_host_vulns(
                        workspace_name, host["ip"]
                    )
                    vulns_data = [
                        OrderedDict(
                            {
                                "ID": x["id"],
                                "NAME": x["value"]["name"],
                                "SEVERITY": style(
                                    x["value"]["severity"].upper(),
                                    fg=utils.get_severity_color(
                                        x["value"]["severity"]
                                    ),
                                ),
                                "STATUS": x["value"]["status"],
                                "CONFIRMED": x["value"]["confirmed"],
                                "TOOL": x["value"]["metadata"]["creator"],
                            }
                        )
                        for x in vulns["vulnerabilities"]
                    ]
                    self._cmd.poutput("\nVulnerabilities:")
                    self._cmd.poutput(
                        tabulate(
                            vulns_data,
                            headers="keys",
                            tablefmt=self._cmd.TABLE_PRETTY_FORMAT
                            if args.pretty
                            else "simple",
                        )
                    )

    delete_host_parser = argparse.ArgumentParser()
    delete_host_parser.add_argument("host_id", type=int, help="ID of the host")
    delete_host_parser.add_argument(
        "-w",
        "--workspace-name",
        type=str,
        help="Workspace name",
        required=False,
    )

    @with_argparser(delete_host_parser)
    def do_delete_host(self, args):
        """Delete Host"""
        if not args.workspace_name:
            if active_config.workspace:
                workspace_name = active_config.workspace
            else:
                self._cmd.perror("No active Workspace")
                return
        else:
            workspace_name = args.workspace_name
        try:
            self._cmd.api_client.delete_host(workspace_name, args.host_id)
        except NotFoundError:
            self._cmd.perror("Host not found")
        except Exception as e:
            self._cmd.perror(f"{e}")
        else:
            self._cmd.poutput(style("Host deleted", fg="green"))

    create_host_parser = argparse.ArgumentParser()
    create_host_parser.add_argument(
        "-d",
        "--host-data",
        type=str,
        help=f"Host data in json format: {HOST_CREATE_JSON_SCHEMA}",
    )
    create_host_parser.add_argument(
        "--stdin", action="store_true", help="Read host-data from stdin"
    )
    create_host_parser.add_argument(
        "-w",
        "--workspace-name",
        type=str,
        help="Workspace name",
        required=False,
    )

    @with_argparser(create_host_parser)
    def do_create_hosts(self, args):
        """Create Hosts"""
        if not args.workspace_name:
            if active_config.workspace:
                workspace_name = active_config.workspace
            else:
                self._cmd.perror("No active Workspace")
                return
        else:
            workspace_name = args.workspace_name
        if args.stdin:
            host_data = sys.stdin.read()
        else:
            if not args.host_data:
                self._cmd.perror("Missing host data")
                return
            else:
                host_data = args.host_data
        try:
            json_data = utils.json_schema_validator(HOST_CREATE_JSON_SCHEMA)(
                host_data
            )
        except Exception as e:
            self._cmd.perror(f"{e}")
        else:
            for _host_data in json_data:
                ip, hostname = utils.get_ip_and_hostname(_host_data["ip"])
                _host_data["ip"] = ip
                if hostname:
                    if "hostnames" in _host_data:
                        _host_data["hostnames"].append(hostname)
                    else:
                        _host_data["hostnames"] = [hostname]
                try:
                    host = self._cmd.api_client.create_host(
                        workspace_name, _host_data
                    )
                except Exception as e:
                    self._cmd.perror(f"{e}")
                else:
                    self._cmd.poutput(
                        f"Created host\n{json.dumps(host, indent=4)}"
                    )



import argparse
import json
import os
import time
from pathlib import Path
import urllib.parse

from faraday_cli.api_client.filter import FaradayFilter
from tabulate import tabulate
from cmd2 import with_argparser, with_default_category, CommandSet

from faraday_cli.extras.halo.halo import Halo
from faraday_cli.config import active_config
from faraday_cli.shell.utils import IGNORE_SEVERITIES, SEVERITIES


@with_default_category("Executive Reports")
class ExecutiveReportsCommands(CommandSet):
    def __init__(self):
        super().__init__()

    report_parser = argparse.ArgumentParser()
    report_parser.add_argument(
        "-w", "--workspace-name", type=str, help="Workspace name"
    )
    report_parser.add_argument(
        "-p",
        "--pretty",
        action="store_true",
        help="Show table in a pretty format",
    )

    @with_argparser(report_parser)
    def do_list_executive_reports_templates(self, args):
        """List executive report templates"""
        if not args.workspace_name:
            if active_config.workspace:
                workspace_name = active_config.workspace
            else:
                self._cmd.perror("No active Workspace")
                return
        else:
            workspace_name = args.workspace_name

        templates_data = self._cmd.api_client.get_executive_report_templates(
            workspace_name
        )
        data = []
        for template in sorted(templates_data["items"], key=lambda x: x[1]):
            data.append({"NAME": template[1], "GROUPED": template[0]})
        self._cmd.poutput(
            tabulate(
                data,
                headers="keys",
                tablefmt=self._cmd.TABLE_PRETTY_FORMAT
                if args.pretty
                else "simple",
            )
        )

    report_parser = argparse.ArgumentParser()
    report_parser.add_argument(
        "-w", "--workspace-name", type=str, help="Workspace name"
    )
    report_parser.add_argument(
        "-t", "--template", type=str, help="Template", required=True
    )
    report_parser.add_argument(
        "--title", type=str, help="Report title", default=""
    )
    report_parser.add_argument(
        "--summary", type=str, help="Report summary", default=""
    )
    report_parser.add_argument(
        "--enterprise", type=str, help="Enterprise name", default=""
    )
    report_parser.add_argument(
        "--confirmed", action="store_true", help="Confirmed vulnerabilities"
    )
    report_parser.add_argument(
        "--severity",
        type=str,
        help=f"Filter by severity {'/'.join(SEVERITIES)}",
        default=[],
        nargs="*",
    )
    report_parser.add_argument(
        "--ignore-info",
        action="store_true",
        help=f"Ignore {'/'.join(IGNORE_SEVERITIES)} vulnerabilities",
    )
    report_parser.add_argument(
        "-o", "--output", type=str, help="Report output"
    )

    @with_argparser(report_parser)
    def do_generate_executive_report(self, args):
        """Generate executive report"""

        @Halo(
            text="Generating executive report",
            text_color="green",
            spinner="dots",
        )
        def get_report(report_data, _output):
            report_id = self._cmd.api_client.generate_executive_report(
                workspace_name, report_data
            )
            report_status = "processing"
            while report_status == "processing":
                time.sleep(2)
                report_status = (
                    self._cmd.api_client.get_executive_report_status(
                        workspace_name, report_id
                    )
                )

            else:
                if report_status == "created":
                    download_response = (
                        self._cmd.api_client.download_executive_report(
                            workspace_name, report_id
                        )
                    )
                    report_file = download_response.headers["x-filename"]
                    if _output:
                        output = Path(_output)
                        if output.is_absolute() and not output.is_dir():
                            output_file = _output
                        else:
                            if output.is_dir():
                                output_file = output / report_file
                            else:
                                output_file = Path(os.getcwd()) / _output
                    else:
                        output_file = Path(os.getcwd()) / report_file
                    with open(output_file, "wb") as f:
                        f.write(download_response.body)
                    return output_file
                else:
                    self._cmd.perror("Error generating executive report")

        if not args.workspace_name:
            if active_config.workspace:
                workspace_name = active_config.workspace
            else:
                self._cmd.perror("No active Workspace")
                return
        else:
            workspace_name = args.workspace_name
        templates_data = self._cmd.api_client.get_executive_report_templates(
            workspace_name
        )
        templates = {
            template[1]: template[0] for template in templates_data["items"]
        }
        report_name = "_".join(
            filter(None, [workspace_name, args.title, args.enterprise])
        )
        if args.template not in templates:
            self._cmd.perror(f"Invalid template: {args.template}")
        else:
            report_data = {
                "conclusions": "",
                "confirmed": args.confirmed,
                "enterprise": args.enterprise,
                "grouped": templates[args.template],
                "name": report_name,
                "objectives": "",
                "recommendations": "",
                "scope": "",
                "summary": args.summary,
                "tags": [],
                "template_name": args.template,
                "title": args.title,
                "vuln_count": 0,
            }
            if args.severity and args.ignore_info:
                self._cmd.perror("Use either --ignore-info or --severity")
                return
            query_filter = FaradayFilter()
            selected_severities = set(map(lambda x: x.lower(), args.severity))
            if selected_severities:
                for severity in selected_severities:
                    if severity not in SEVERITIES:
                        self._cmd.perror(f"Invalid severity: {severity}")
                        return
                    else:
                        query_filter.require_severity(severity)
            if args.ignore_info:
                for severity in IGNORE_SEVERITIES:
                    query_filter.ignore_severity(severity)
            if args.confirmed:
                query_filter.filter_confirmed()
            report_data["filter"] = urllib.parse.quote(
                json.dumps(query_filter.get_filter())
            )
            report_file = get_report(report_data, args.output)
            self._cmd.poutput(f"Report generated: {report_file}")



import json
import argparse
import sys
from collections import OrderedDict

from cmd2 import style, with_argparser, with_default_category, CommandSet
from tabulate import tabulate
from simple_rest_client.exceptions import NotFoundError

from faraday_cli.config import active_config
from faraday_cli.shell import utils
from faraday_cli.shell.exceptions import InvalidJson, InvalidJsonSchema


@with_default_category("Agents")
class AgentCommands(CommandSet):
    def __init__(self):
        super().__init__()

    list_agent_parser = argparse.ArgumentParser()
    list_agent_parser.add_argument(
        "-w", "--workspace-name", type=str, help="Workspace name"
    )
    list_agent_parser.add_argument(
        "-j", "--json-output", action="store_true", help="Show output in json"
    )
    list_agent_parser.add_argument(
        "-p",
        "--pretty",
        action="store_true",
        help="Show table in a pretty format ",
    )

    @with_argparser(list_agent_parser)
    def do_list_agents(self, args):
        """List agents"""
        if not args.workspace_name:
            if active_config.workspace:
                workspace_name = active_config.workspace
            else:
                self._cmd.perror("No active Workspace")
                return
        else:
            workspace_name = args.workspace_name
        try:
            agents = self._cmd.api_client.get_workspace_agents(workspace_name)
        except NotFoundError:
            self._cmd.perror("Workspace not found")
        else:
            if not agents:
                self._cmd.perror(f"No agents in workspace: {workspace_name}")
            else:
                if args.json_output:
                    self._cmd.poutput(json.dumps(agents, indent=4))
                else:
                    data = [
                        OrderedDict(
                            {
                                "ID": x["id"],
                                "NAME": x["name"],
                                "ACTIVE": x["active"],
                                "STATUS": x["status"],
                                "EXECUTORS": ", ".join(
                                    [i["name"] for i in x["executors"]]
                                ),
                            }
                        )
                        for x in agents
                    ]

                    self._cmd.poutput(
                        tabulate(
                            data,
                            headers="keys",
                            tablefmt="psql" if args.pretty else "simple",
                        )
                    )

    get_agent_parser = argparse.ArgumentParser()
    get_agent_parser.add_argument("agent_id", type=int, help="ID of the Agent")
    get_agent_parser.add_argument(
        "-w", "--workspace-name", type=str, help="Workspace name "
    )
    get_agent_parser.add_argument(
        "-j", "--json-output", action="store_true", help="Show output in json"
    )
    get_agent_parser.add_argument(
        "-p",
        "--pretty",
        action="store_true",
        help="Show table in a pretty format",
    )

    @with_argparser(get_agent_parser)
    def do_get_agent(self, args):
        """Get agent"""
        if not args.workspace_name:
            if active_config.workspace:
                workspace_name = active_config.workspace
            else:
                self._cmd.perror("No active Workspace")
                return
        else:
            workspace_name = args.workspace_name
        try:
            agent = self._cmd.api_client.get_agent(
                workspace_name, args.agent_id
            )
        except NotFoundError:
            self._cmd.perror("Agent not found")
        else:
            if args.json_output:
                self._cmd.poutput(json.dumps(agent, indent=4))
            else:
                data = [
                    OrderedDict(
                        {
                            "ID": x["id"],
                            "NAME": x["name"],
                            "ACTIVE": x["active"],
                            "STATUS": x["status"],
                        }
                    )
                    for x in [agent]
                ]
                executors_data = [
                    OrderedDict(
                        {
                            "ID": x["id"],
                            "NAME": x["name"],
                            "PARAMETERS [REQUIRED]": "".join(
                                [
                                    f"{parameter} [{required}]\n"
                                    for parameter, required in x[
                                        "parameters_metadata"
                                    ].items()
                                ]
                            ),
                        }
                    )
                    for x in agent["executors"]
                ]

                self._cmd.poutput(
                    tabulate(
                        data,
                        headers="keys",
                        tablefmt="psql" if args.pretty else "simple",
                    )
                )
                self._cmd.poutput("\nExecutors:")
                self._cmd.poutput(
                    tabulate(
                        executors_data,
                        headers="keys",
                        tablefmt="psql" if args.pretty else "simple",
                    )
                )

    run_executor_parser = argparse.ArgumentParser()
    run_executor_parser.add_argument(
        "-a", "--agent-id", type=int, help="ID of the agent", required=True
    )
    run_executor_parser.add_argument(
        "-e", "--executor-name", type=str, help="Executor name", required=True
    )
    run_executor_parser.add_argument(
        "-p",
        "--executor-params",
        type=str,
        help="Executor Params in json",
    )
    run_executor_parser.add_argument(
        "--stdin", action="store_true", help="Read executor-params from stdin"
    )
    run_executor_parser.add_argument(
        "-w", "--workspace-name", type=str, help="Workspace name"
    )

    @with_argparser(run_executor_parser)
    def do_run_executor(self, args):
        """Run executor"""
        if args.stdin:
            executor_params = sys.stdin.read()
        else:
            if not args.executor_params:
                self._cmd.perror("Missing executor params")
                return
            else:
                executor_params = args.executor_params
        if not args.workspace_name:
            if active_config.workspace:
                workspace_name = active_config.workspace
            else:
                self._cmd.perror("No active Workspace")
                return
        else:
            workspace_name = args.workspace_name
        try:
            utils.validate_json(args.executor_params)
        except InvalidJson as e:
            self._cmd.perror(e)
        else:
            try:
                agent = self._cmd.api_client.get_agent(
                    workspace_name, args.agent_id
                )
            except NotFoundError:
                self._cmd.perror("Agent not found")
            else:
                executor = None
                for executor in agent["executors"]:
                    if executor["name"] == args.executor_name:
                        break
                if not executor:
                    self._cmd.perror(
                        f"Invalid executor name [{args.executor_name}]"
                    )
                    return
                executor_parameters_schema = {
                    "type": "object",
                    "properties": {
                        x: {"type": "string"}
                        for x in executor["parameters_metadata"].keys()
                    },
                    "required": [
                        i[0]
                        for i in filter(
                            lambda x: x[1] is True,
                            executor["parameters_metadata"].items(),
                        )
                    ],
                }
                try:
                    utils.json_schema_validator(executor_parameters_schema)(
                        executor_params
                    )
                except InvalidJsonSchema as e:
                    self._cmd.perror(e)
                else:
                    try:
                        response = self._cmd.api_client.run_executor(
                            workspace_name,
                            args.agent_id,
                            executor["name"],
                            json.loads(executor_params),
                        )
                    except Exception as e:
                        print(e)
                    else:
                        self._cmd.poutput(
                            style(
                                f"Run executor: {agent['name']}/{executor['name']} [{response}]",  # noqa: E501
                                fg="green",
                            )
                        )



import json
import sys
import argparse
from collections import OrderedDict

from cmd2 import with_argparser, with_default_category, CommandSet
from tabulate import tabulate
from simple_rest_client.exceptions import NotFoundError

from faraday_cli.extras.halo.halo import Halo
from faraday_cli.config import active_config


HOST_CREATE_JSON_SCHEMA = {
    "type": "array",
    "items": {
        "type": "object",
        "properties": {
            "ip": {"type": "string"},
            "description": {"type": "string"},
            "hostnames": {"type": "array"},
        },
        "required": ["ip", "description"],
    },
}


@with_default_category("Services")
class ServiceCommands(CommandSet):
    def __init__(self):
        super().__init__()

    list_service_parser = argparse.ArgumentParser()
    list_service_parser.add_argument(
        "-w", "--workspace-name", type=str, help="Workspace"
    )
    list_service_parser.add_argument(
        "-j", "--json-output", action="store_true", help="JSON output"
    )
    list_service_parser.add_argument(
        "-p", "--pretty", action="store_true", help="Pretty Tables"
    )

    @with_argparser(list_service_parser)
    def do_list_services(self, args):
        """List services"""

        @Halo(
            text="Gathering data",
            text_color="green",
            spinner="dots",
            stream=sys.stderr,
        )
        def get_data(workspace_name):
            services = self._cmd.api_client.get_services(workspace_name)
            for i in services["services"]:
                i["value"]["host"] = get_service_host(i["value"]["host_id"])
            return services

        host_cache = {}

        def get_service_host(host_id):
            if host_id not in host_cache:
                host_cache[host_id] = self._cmd.api_client.get_host(
                    workspace_name, host_id
                )
            return host_cache[host_id]

        if not args.workspace_name:
            if active_config.workspace:
                workspace_name = active_config.workspace
            else:
                self._cmd.perror("No active Workspace")
                return
        else:
            workspace_name = args.workspace_name
        try:
            services = get_data(workspace_name)
        except NotFoundError:
            self._cmd.perror("Workspace not found")
        else:
            if not services["services"]:
                self._cmd.perror(f"No services in workspace: {workspace_name}")
            else:
                if args.json_output:
                    self._cmd.poutput(
                        json.dumps(services["services"], indent=4)
                    )
                else:

                    data = [
                        OrderedDict(
                            {
                                "ID": x["id"],
                                "NAME": x["value"]["name"],
                                "SUMMARY": x["value"]["summary"],
                                "IP": x["value"]["host"]["ip"],
                                "PORT": x["value"]["port"],
                                "PROTOCOL": x["value"]["protocol"],
                                "HOST": x["value"]["host_id"],
                                "VULNERABILITIES": x["value"]["vulns"],
                            }
                        )
                        for x in sorted(
                            services["services"],
                            key=lambda x: x["value"]["host_id"],
                        )
                    ]
                    self._cmd.poutput(
                        tabulate(
                            data,
                            headers="keys",
                            tablefmt="psql" if args.pretty else "simple",
                        )
                    )



import argparse
import sys

from faraday_cli.shell.shell import FaradayShell


def main(argv=None):

    parser = argparse.ArgumentParser(description="Commands as arguments")
    command_help = (
        "optional command to run, "
        "if no command given, enter an interactive shell"
    )
    parser.add_argument("command", nargs="?", help=command_help)
    arg_help = "optional arguments for command"
    parser.add_argument(
        "command_args", nargs=argparse.REMAINDER, help=arg_help
    )

    args = parser.parse_args(argv)
    app = FaradayShell()
    sys_exit_code = 0
    if args.command:
        # we have a command, run it and then exit
        command_string = f"{args.command} {' '.join(args.command_args)}"
        app.onecmd_plus_hooks(command_string)
    else:
        # we have no command, drop into interactive mode
        app.run_as_shell = True
        sys_exit_code = app.cmdloop()
    sys.exit(sys_exit_code)


if __name__ == "__main__":
    main()



from simple_rest_client.resource import Resource


class LoginResource(Resource):
    actions = {
        "auth": {"method": "POST", "url": "login"},
        "get_token": {"method": "GET", "url": "v2/token/"},
        "validate": {"method": "GET", "url": "v2/preferences/"},
        "second_factor": {"method": "POST", "url": "confirmation"},
    }


class ConfigResource(Resource):
    actions = {
        "config": {"method": "GET", "url": "config"},
    }


class WorkspaceResource(Resource):
    actions = {
        "list": {"method": "GET", "url": "v2/ws/"},
        "get": {"method": "GET", "url": "v2/ws/{}/"},
        "create": {"method": "POST", "url": "v2/ws/"},
        "delete": {"method": "DELETE", "url": "v2/ws/{}/"},
    }


class BulkCreateResource(Resource):
    actions = {"create": {"method": "POST", "url": "v2/ws/{}/bulk_create/"}}


class HostResource(Resource):
    actions = {
        "list": {"method": "GET", "url": "v2/ws/{}/hosts/"},
        "get": {"method": "GET", "url": "v2/ws/{}/hosts/{}/"},
        "create": {"method": "POST", "url": "v2/ws/{}/hosts/"},
        "delete": {"method": "DELETE", "url": "v2/ws/{}/hosts/{}/"},
        "get_services": {
            "method": "GET",
            "url": "v2/ws/{}/hosts/{}/services/",
        },
        "get_vulns": {"method": "GET", "url": "v2/ws/{}/vulns/"},
    }


class ServiceResource(Resource):
    actions = {
        "list": {"method": "GET", "url": "v2/ws/{}/services/"},
        "get": {"method": "GET", "url": "v2/ws/{}/services/{}/"},
    }


class VulnResource(Resource):
    actions = {
        "list": {"method": "GET", "url": "v2/ws/{}/vulns/"},
        "filter": {"method": "GET", "url": "v2/ws/{}/vulns/filter"},
    }


class CredentialResource(Resource):
    actions = {
        "list": {"method": "GET", "url": "v2/ws/{}/credential/"},
    }


class AgentResource(Resource):
    actions = {
        "list": {"method": "GET", "url": "v2/ws/{}/agents/"},
        "get": {"method": "GET", "url": "v2/ws/{}/agents/{}/"},
        "run": {"method": "POST", "url": "v2/ws/{}/agents/{}/run/"},
    }


class ExecutiveReportResource(Resource):
    actions = {
        "list_templates": {
            "method": "GET",
            "url": "v2/ws/{}/reports/listTemplates/",
        },
        "generate": {"method": "POST", "url": "v2/ws/{}/reports/"},
        "status": {"method": "GET", "url": "v2/ws/{}/reports/{}/"},
        "download": {"method": "GET", "url": "v2/ws/{}/reports/{}/download/"},
    }



import json
import argparse
from collections import OrderedDict

from cmd2 import style, with_argparser, with_default_category, CommandSet
from simple_rest_client.exceptions import NotFoundError
from tabulate import tabulate

from faraday_cli.config import active_config


@with_default_category("Workspaces")
class WorkspaceCommands(CommandSet):
    def __init__(self):
        super().__init__()

    select_ws_parser = argparse.ArgumentParser()
    select_ws_parser.add_argument(
        "workspace_name", type=str, help="Workspace name"
    )

    @with_argparser(select_ws_parser)
    def do_select_ws(self, args):
        """Select active Workspace"""
        if self._cmd.api_client.is_workspace_valid(args.workspace_name):
            active_config.workspace = args.workspace_name
            active_config.save()
            self._cmd.poutput(
                style(
                    f"{self._cmd.emojis['check']} "
                    f"Selected workspace: {args.workspace_name}",
                    fg="green",
                )
            )
            self._cmd.update_prompt()
        else:
            self._cmd.perror(
                f"{self._cmd.emojis['cross']} "
                f"Invalid workspace: {args.workspace_name}"
            )

    list_ws_parser = argparse.ArgumentParser()
    list_ws_parser.add_argument(
        "-j", "--json-output", action="store_true", help="Show output in json"
    )
    list_ws_parser.add_argument(
        "-p",
        "--pretty",
        action="store_true",
        help="Show table in a pretty format",
    )

    @with_argparser(list_ws_parser)
    def do_list_ws(self, args):
        """List Workspaces"""
        workspaces = self._cmd.api_client.get_workspaces()
        if args.json_output:
            self._cmd.poutput(json.dumps(workspaces, indent=4))
            return
        else:
            if not workspaces:
                self._cmd.poutput("No workspaces available")
                return
            else:
                data = [
                    OrderedDict(
                        {
                            "NAME": x["name"],
                            "HOSTS": x["stats"]["hosts"],
                            "SERVICES": x["stats"]["services"],
                            "VULNS": "-"
                            if x["stats"]["total_vulns"] == 0
                            else x["stats"]["total_vulns"],
                            "ACTIVE": x["active"],
                            "PUBLIC": x["public"],
                            "READONLY": x["readonly"],
                        }
                    )
                    for x in workspaces
                ]
                self._cmd.poutput(
                    tabulate(
                        data,
                        headers="keys",
                        tablefmt=self._cmd.TABLE_PRETTY_FORMAT
                        if args.pretty
                        else "simple",
                    )
                )

    get_ws_parser = argparse.ArgumentParser()
    get_ws_parser.add_argument(
        "-w",
        "--workspace-name",
        type=str,
        help="Workspace name",
        required=False,
    )
    get_ws_parser.add_argument(
        "-j", "--json-output", action="store_true", help="Show output in json"
    )
    get_ws_parser.add_argument(
        "-p",
        "--pretty",
        action="store_true",
        help="Show table in a pretty format",
    )

    @with_argparser(get_ws_parser)
    def do_get_ws(self, args):
        """Get Workspace"""
        if not args.workspace_name:
            if active_config.workspace:
                workspace_name = active_config.workspace
            else:
                self._cmd.perror("No active Workspace")
                return
        else:
            workspace_name = args.workspace_name
        try:
            workspace = self._cmd.api_client.get_workspace(workspace_name)
        except NotFoundError:
            self._cmd.perror(f"Workspace {workspace_name} not found")
        else:
            if args.json_output:
                self._cmd.poutput(json.dumps(workspace, indent=4))
                return
            else:
                data = [
                    {
                        "NAME": workspace["name"],
                        "ACTIVE": workspace["active"],
                        "PUBLIC": workspace["public"],
                        "READONLY": workspace["readonly"],
                        "HOSTS": "-"
                        if workspace["stats"]["hosts"] == 0
                        else workspace["stats"]["hosts"],
                        "SERVICES": "-"
                        if workspace["stats"]["services"] == 0
                        else workspace["stats"]["services"],
                        "VULNS": "-"
                        if workspace["stats"]["total_vulns"] == 0
                        else workspace["stats"]["total_vulns"],
                    }
                ]
                self._cmd.poutput(
                    tabulate(
                        data,
                        headers="keys",
                        tablefmt=self._cmd.TABLE_PRETTY_FORMAT
                        if args.pretty
                        else "simple",
                    )
                )

    delete_ws_parser = argparse.ArgumentParser()
    delete_ws_parser.add_argument(
        "workspace_name", type=str, help="Workspace name"
    )

    @with_argparser(delete_ws_parser)
    def do_delete_ws(self, args):
        """Delete Workspace"""
        workspaces = self._cmd.api_client.get_workspaces()
        workspace_choices = [ws for ws in map(lambda x: x["name"], workspaces)]
        workspace_name = args.workspace_name
        if workspace_name not in workspace_choices:
            self._cmd.perror(f"Invalid workspace: {workspace_name}")
            return
        self._cmd.poutput(f"Deleting workspace: {workspace_name}")
        self._cmd.api_client.delete_workspace(workspace_name)
        self._cmd.poutput(
            style(f"Deleted workspace: {args.workspace_name}", fg="green")
        )
        if active_config.workspace == workspace_name:
            active_config.workspace = None
            active_config.save()
        self._cmd.update_prompt()

    create_ws_parser = argparse.ArgumentParser()
    create_ws_parser.add_argument(
        "workspace_name", type=str, help="Workspace name"
    )
    create_ws_parser.add_argument(
        "-d",
        "--dont-select",
        action="store_true",
        help="Dont select after create",
    )

    @with_argparser(create_ws_parser)
    def do_create_ws(self, args):
        """Create Workspace"""
        workspace_name = args.workspace_name
        try:
            self._cmd.api_client.create_workspace(workspace_name)
        except Exception as e:
            self._cmd.perror(f"{e}")
        else:
            self._cmd.poutput(
                style(
                    f"{self._cmd.emojis['check']} "
                    f"Created workspace: {args.workspace_name}",
                    fg="green",
                )
            )
            if not args.dont_select:
                active_config.workspace = workspace_name
                active_config.save()
                self._cmd.update_prompt()



from pathlib import Path
import argparse
import getpass

from cmd2 import with_argparser, with_default_category, CommandSet, style
from faraday_cli.config import active_config


@with_default_category("Tools Reports")
class ReportsCommands(CommandSet):
    def __init__(self):
        super().__init__()

    report_parser = argparse.ArgumentParser()
    report_parser.add_argument(
        "-w", "--workspace-name", type=str, help="Workspace"
    )
    report_parser.add_argument(
        "--plugin-id",
        type=str,
        help="Plugin ID (force detection)",
        required=False,
    )
    report_parser.add_argument("report_path", help="Path of the report file")

    @with_argparser(report_parser, preserve_quotes=True)
    def do_process_report(self, args):
        """Process Tool report in Faraday"""
        report_path = Path(args.report_path)
        if not report_path.is_file():
            self._cmd.perror(f"File {report_path} dont exists")
            return
        if not args.workspace_name:
            if active_config.workspace:
                workspace_name = active_config.workspace
            else:
                self._cmd.perror("No active Workspace")
                return
        else:
            workspace_name = args.workspace_name
        if not self._cmd.api_client.is_workspace_valid(workspace_name):
            self._cmd.perror(f"Invalid workspace: {workspace_name}")
            return
        else:
            destination_workspace = workspace_name
        if args.plugin_id:
            plugin = self._cmd.plugins_manager.get_plugin(args.plugin_id)
            if not plugin:
                self._cmd.perror(f"Invalid Plugin: {args.plugin_id}")
                return
        else:
            plugin = self._cmd.report_analyzer.get_plugin(report_path)
            if not plugin:
                self._cmd.perror(
                    f"{self._cmd.emojis['cross']} "
                    f"Failed to detect report: {report_path}"
                )
                return
        self._cmd.poutput(
            style(
                f"{self._cmd.emojis['page']} Processing {plugin.id} report",
                fg="green",
            )
        )
        plugin.processReport(
            report_path.absolute().as_posix(), getpass.getuser()
        )
        self._cmd.data_queue.put(
            {
                "workspace": destination_workspace,
                "json_data": plugin.get_data(),
            }
        )



import os
import yaml
from pathlib import Path

DEFAULT_CONFIG_FILE = os.environ.get(
    "FARADAY_CLI_CONFIG", "~/.faraday-cli.yml"
)


class Config:
    def __init__(self):
        self.config_file = Path(os.path.expanduser(DEFAULT_CONFIG_FILE))
        self.faraday_url = None
        self.token = None
        self.ignore_ssl = False
        self.workspace = None
        self.custom_plugins_folder = None
        self.load()

    def config_exists(self) -> bool:
        return self.config_file.exists()

    def load(self):
        try:
            if self.config_exists():
                with open(self.config_file) as f:
                    config_data = yaml.load(f, Loader=yaml.FullLoader)
                self.faraday_url = config_data.get("auth", {}).get(
                    "faraday_url"
                )
                self.token = config_data.get("auth", {}).get("token")
                self.ignore_ssl = config_data.get("auth", {}).get("ignore_ssl")
                self.workspace = config_data.get("faraday", {}).get(
                    "workspace"
                )
                self.custom_plugins_folder = config_data.get(
                    "settings", {}
                ).get("custom_plugins_folder")
        except KeyError:
            pass

    def save(self):
        config_data = {
            "auth": {
                "faraday_url": self.faraday_url,
                "token": self.token,
                "ignore_ssl": self.ignore_ssl,
            },
            "faraday": {"workspace": self.workspace},
            "settings": {"custom_plugins_folder": self.custom_plugins_folder},
        }
        with open(self.config_file, "w") as f:
            yaml.dump(config_data, f)


active_config = Config()



import os
import re
from urllib.parse import urljoin

import click
from faraday_cli.api_client.exceptions import (
    DuplicatedError,
    InvalidCredentials,
    Invalid2FA,
    MissingConfig,
)
from simple_rest_client.api import API


from faraday_cli.api_client import resources, exceptions
from simple_rest_client.exceptions import (
    AuthError,
    NotFoundError,
    ClientError,
    ClientConnectionError,
)

DEFAULT_TIMEOUT = int(os.environ.get("FARADAY_CLI_TIMEOUT", 10000))


class FaradayApi:
    def __init__(self, url=None, ignore_ssl=False, token=None):
        if url:
            self.api_url = urljoin(url, "_api")
        else:
            self.api_url = None
        self.token = token
        if self.token:
            headers = {"Authorization": f"Token {self.token}"}
        else:
            headers = {}
        ssl_verify = not ignore_ssl
        self.faraday_api = API(
            api_root_url=self.api_url,
            params={},
            headers=headers,
            timeout=DEFAULT_TIMEOUT,
            append_slash=False,
            json_encode_body=True,
            ssl_verify=ssl_verify,
        )
        self._build_resources()

    def handle_errors(func):
        def hanlde(self, *args, **kwargs):
            if not self.token:
                raise MissingConfig("Missing Config, run 'faraday-cli auth'")
            try:
                result = func(self, *args, **kwargs)
            except InvalidCredentials:
                raise
            except AuthError:
                raise InvalidCredentials(
                    "Invalid credentials, run 'faraday-cli auth'"
                )
            except ClientConnectionError as e:
                raise Exception(
                    f"Connection error: {e} (check you faraday server)"
                )
            except DuplicatedError as e:
                raise Exception(f"{e}")
            except NotFoundError:
                raise
            except ClientError:
                raise
            except Exception as e:
                raise Exception(f"Unknown error: {type(e)} - {e}")
            else:
                return result

        return hanlde

    def _build_resources(self):
        self.faraday_api.add_resource(
            resource_name="login", resource_class=resources.LoginResource
        )
        self.faraday_api.add_resource(
            resource_name="config", resource_class=resources.ConfigResource
        )
        self.faraday_api.add_resource(
            resource_name="workspace",
            resource_class=resources.WorkspaceResource,
        )
        self.faraday_api.add_resource(
            resource_name="bulk_create",
            resource_class=resources.BulkCreateResource,
        )
        self.faraday_api.add_resource(
            resource_name="host", resource_class=resources.HostResource
        )
        self.faraday_api.add_resource(
            resource_name="service", resource_class=resources.ServiceResource
        )
        self.faraday_api.add_resource(
            resource_name="credential",
            resource_class=resources.CredentialResource,
        )
        self.faraday_api.add_resource(
            resource_name="agent", resource_class=resources.AgentResource
        )
        self.faraday_api.add_resource(
            resource_name="vuln", resource_class=resources.VulnResource
        )
        self.faraday_api.add_resource(
            resource_name="executive_report",
            resource_class=resources.ExecutiveReportResource,
        )

    def login(self, user, password):
        body = {"email": user, "password": password}
        try:
            response = self.faraday_api.login.auth(body=body)
            if response.status_code == 202:
                return None
        except NotFoundError:
            raise
        except AuthError:
            return False
        except ClientConnectionError:
            raise
        else:
            return True

    def get_token(self, second_factor=None):
        if not self.token:
            try:
                # self.faraday_api.login.auth(body=login_body)
                if second_factor:
                    second_factor_body = {"secret": second_factor}
                    try:
                        self.faraday_api.login.second_factor(
                            body=second_factor_body
                        )
                    except AuthError:
                        raise Invalid2FA("Invalid 2FA")
                token_response = self.faraday_api.login.get_token()
            except NotFoundError:
                # raise Exception(
                #    f"Invalid url: {self.faraday_api.api_root_url}"
                # )
                raise
            except AuthError:
                raise InvalidCredentials()
            except ClientConnectionError:
                raise
            else:
                self.token = token_response.body
        return self.token

    @handle_errors
    def is_token_valid(self):
        try:
            self.faraday_api.login.validate()
        except ClientConnectionError as e:
            raise click.ClickException(
                click.style(f"Connection to error: {e}", fg="red")
            )
        except AuthError:
            return False
        else:
            return True

    @handle_errors
    def get_version(self):
        version_regex = r"(?P<product>\w)?-?(?P<version>\d+\.\d+)"
        response = self.faraday_api.config.config()
        raw_version = response.body["ver"]
        match = re.match(version_regex, raw_version)
        products = {"p": "pro", "c": "corp"}
        product = products.get(match.group("product"), "community")
        version = match.group("version")
        return {"product": product, "version": version}

    @handle_errors
    def get_workspaces(self):
        response = self.faraday_api.workspace.list()
        return response.body

    @handle_errors
    def get_workspace(self, workspace_name: str):
        response = self.faraday_api.workspace.get(workspace_name)
        return response.body

    @handle_errors
    def get_hosts(self, workspace_name: str, port: int = None):
        if port:
            response = self.faraday_api.host.list(
                workspace_name, params={"port": port}
            )
        else:
            response = self.faraday_api.host.list(workspace_name)
        return response.body

    @handle_errors
    def get_services(self, workspace_name: str):
        response = self.faraday_api.service.list(workspace_name)
        return response.body

    @handle_errors
    def get_vulns(self, workspace_name: str, query_filter: dict = None):
        if query_filter:
            # encoded_query = urllib.parse.urlencode({'q': query_filter})
            response = self.faraday_api.vuln.filter(
                workspace_name, params={"q": query_filter}
            )
        else:
            response = self.faraday_api.vuln.list(workspace_name)
        return response.body

    @handle_errors
    def get_workspace_credentials(self, workspace_name: str):
        response = self.faraday_api.credential.list(workspace_name)
        return response.body

    @handle_errors
    def get_workspace_agents(self, workspace_name: str):
        response = self.faraday_api.agent.list(workspace_name)
        return response.body

    @handle_errors
    def get_agent(self, workspace_name: str, agent_id: int):
        response = self.faraday_api.agent.get(workspace_name, agent_id)
        return response.body

    @handle_errors
    def run_executor(self, workspace_name: str, agent_id, executor_name, args):
        body = {
            "executorData": {
                "agent_id": agent_id,
                "args": args,
                "executor": executor_name,
            }
        }
        response = self.faraday_api.agent.run(
            workspace_name, agent_id, body=body
        )
        return response.body

    @handle_errors
    def get_host(self, workspace_name: str, host_id):
        response = self.faraday_api.host.get(workspace_name, host_id)
        return response.body

    @handle_errors
    def delete_host(self, workspace_name: str, host_id):
        response = self.faraday_api.host.delete(workspace_name, host_id)
        return response.body

    @handle_errors
    def create_host(self, workspace_name: str, host_params):
        try:
            response = self.faraday_api.host.create(
                workspace_name, body=host_params
            )
        except ClientError as e:
            if e.response.status_code == 409:
                raise exceptions.DuplicatedError("Host already exist")
            else:
                raise
        else:
            return response.body

    @handle_errors
    def get_host_services(self, workspace_name: str, host_id):
        response = self.faraday_api.host.get_services(workspace_name, host_id)
        return response.body

    @handle_errors
    def get_host_vulns(self, workspace_name: str, host_ip):
        response = self.faraday_api.host.get_vulns(
            workspace_name, params={"target": host_ip}
        )
        return response.body

    @handle_errors
    def bulk_create(self, workspace_name, data):
        response = self.faraday_api.bulk_create.create(
            workspace_name, body=data
        )
        return response.body

    @handle_errors
    def create_workspace(
        self, workspace_name: str, description="", users=None
    ):
        default_users = ["faraday"]
        if users:
            if isinstance(users, str):
                default_users.append(users)
            elif isinstance(users, list):
                default_users.extend(users)
        data = {
            "description": description,
            "id": 0,
            "name": workspace_name,
            "public": False,
            "readonly": False,
            "customer": "",
            "users": default_users,
        }
        try:
            response = self.faraday_api.workspace.create(body=data)
        except ClientError as e:
            if e.response.status_code == 409:
                raise exceptions.DuplicatedError("Workspace already exist")
            else:
                raise
        else:
            return response.body

    @handle_errors
    def delete_workspace(self, workspace_name: str):
        response = self.faraday_api.workspace.delete(workspace_name)
        return response

    @handle_errors
    def is_workspace_valid(self, workspace_name):
        workspaces = self.get_workspaces()
        available_workspaces = [
            ws for ws in map(lambda x: x["name"], workspaces)
        ]
        return workspace_name in available_workspaces

    @handle_errors
    def get_executive_report_templates(self, workspace_name: str):
        response = self.faraday_api.executive_report.list_templates(
            workspace_name
        )
        return response.body

    @handle_errors
    def generate_executive_report(
        self, workspace_name: str, report_data: dict
    ):
        response = self.faraday_api.executive_report.generate(
            workspace_name, body=report_data
        )
        return response.body["id"]

    @handle_errors
    def get_executive_report_status(self, workspace_name: str, report_id: int):
        response = self.faraday_api.executive_report.status(
            workspace_name, report_id
        )
        return response.body["status"]

    @handle_errors
    def download_executive_report(self, workspace_name: str, report_id: int):
        response = self.faraday_api.executive_report.download(
            workspace_name, report_id
        )
        return response



from setuptools import setup, find_packages
from re import search

with open("faraday_cli/__init__.py", "rt", encoding="utf8") as f:
    version = search(r"__version__ = \"(.*?)\"", f.read()).group(1)


install_requires = [
    "Click",
    "colorama",
    "faraday-plugins",
    "jsonschema",
    "PyYAML",
    "simple-rest-client",
    "tabulate",
    "validators",
    "spinners",
    "log_symbols",
    "termcolor",
    "cmd2",
]


extra_req = {
    "dev": ["giteasychangelog", "flake8", "black", "pre-commit"],
    "test": ["pytest", "pytest-cov", "faradaysec"],
    "docs": [
        "mkdocs",
        "mkdocs-material",
    ],
}

with open("README.md") as readme_file:
    readme = readme_file.read()

with open("RELEASE.md") as history_file:
    history = history_file.read()


setup(
    name="faraday-cli",
    version=version,
    packages=find_packages(include=["faraday_cli", "faraday_cli.*"]),
    url="https://github.com/infobyte/faraday-cli",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Natural Language :: English",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
    ],
    license="GNU General Public License v3",
    author="Faradaysec",
    python_requires=">3.7.0",
    author_email="devel@faradaysec.com",
    long_description=readme + "\n\n" + history,
    long_description_content_type="text/markdown",
    description="Faraday cli package",
    include_package_data=True,
    install_requires=install_requires,
    extras_require=extra_req,
    entry_points={
        "console_scripts": ["faraday-cli=faraday_cli.shell.main:main"],
    },
)



from .workspace import WorkspaceCommands  # noqa: F401
from .stats import StatsCommands  # noqa: F401
from .reports import ReportsCommands  # noqa: F401
from .host import HostCommands  # noqa: F401
from .service import ServiceCommands  # noqa: F401
from .agent import AgentCommands  # noqa: F401
from .executive_report import ExecutiveReportsCommands  # noqa: F401



import json
from socket import gethostbyname, inet_aton
import jsonschema

from validators import url

from .exceptions import InvalidJson, InvalidJsonSchema

IGNORE_SEVERITIES = ("informational", "unclassified")
SEVERITIES = (
    "informational",
    "critical",
    "high",
    "medium",
    "low",
    "unclassified",
)
SEVERITY_COLORS = {
    "critical": "magenta",
    "high": "red",
    "med": "yellow",
    "low": "green",
    "info": "blue",
    "unclassified": "cyan",
}


def validate_url(value):
    valid_url = url(value)
    if valid_url:
        return value
    else:
        raise Exception(f"Invalid url: {value}")


def validate_json(value):
    if value:
        try:
            json_value = json.loads(value)
        except Exception as e:
            raise InvalidJson(f"Invalid json parameter: {value} - {e}")
        else:
            return json_value


def json_schema_validator(schema):
    def _validate_json(value):
        if value:
            if isinstance(value, str):
                try:
                    json_value = json.loads(value)
                except Exception as e:
                    raise Exception(f"Invalid json format: {value} - {e}")
            else:
                json_value = value
            try:
                jsonschema.validate(instance=json_value, schema=schema)
            except jsonschema.exceptions.ValidationError as err:
                raise InvalidJsonSchema(f"{err}")
            return json_value

    return _validate_json


def get_ip_and_hostname(value):
    try:
        inet_aton(value)
        return value, None
    except Exception:
        try:
            ip_address = gethostbyname(value)
        except OSError:
            return value, None
        else:
            return ip_address, value


def trim_long_text(text, size=50):
    if len(text) <= size:
        return text
    else:
        return f"{text[:size]}..."


def get_severity_color(severity):
    return SEVERITY_COLORS.get(severity, "white")


def get_ignore_info_severity_filter() -> dict:
    query_filter = {
        "filters": [
            {
                "and": [
                    {"name": "severity", "op": "neq", "val": severity}
                    for severity in IGNORE_SEVERITIES
                ]
            }
        ]
    }
    return query_filter


def get_severity_filter(severities: list) -> dict:
    query_filter = {
        "filters": [
            {
                "or": [
                    {"name": "severity", "op": "eq", "val": severity}
                    for severity in severities
                ]
            }
        ]
    }
    return query_filter


def get_confirmed_filter() -> dict:
    query_filter = {
        "filters": [{"name": "confirmed", "op": "==", "val": "true"}]
    }
    return query_filter



class FaradayFilter:
    def __init__(self):
        self.__status_required = []
        self.__status_not_required = []
        self.__severities_required = []
        self.__severities_ignored = []
        self.__confirmed = False

    def require_severity(self, severity: str) -> None:
        if severity not in self.__severities_required:
            self.__severities_required.append(severity)

    def ignore_severity(self, severity: str) -> None:
        if severity not in self.__severities_ignored:
            self.__severities_ignored.append(severity)

    def filter_confirmed(self) -> None:
        self.__confirmed = True

    def get_filter(self) -> dict:
        filter_data = []

        if self.__severities_required:
            if len(self.__severities_required) > 1:
                filter_data.append(
                    {
                        "or": [
                            {"name": "severity", "op": "eq", "val": value}
                            for value in self.__severities_required
                        ]
                    }
                )
            else:
                filter_data.append(
                    {
                        "name": "severity",
                        "op": "eq",
                        "val": self.__severities_required[0],
                    }
                )

        if self.__severities_ignored:
            if len(self.__severities_ignored) > 1:
                filter_data.append(
                    {
                        "or": [
                            {"name": "severity", "op": "neq", "val": value}
                            for value in self.__severities_ignored
                        ]
                    }
                )
            else:
                filter_data.append(
                    {
                        "name": "severity",
                        "op": "neq",
                        "val": self.__severities_ignored[0],
                    }
                )
        if self.__confirmed:
            if filter_data:
                filter_data.append(
                    {"and": [{"name": "confirmed", "op": "eq", "val": "true"}]}
                )
            else:
                filter_data.append(
                    {"name": "confirmed", "op": "eq", "val": "true"}
                )
        return {"filters": filter_data}
